"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const tempy = require("tempy");
const fs = require("fs");
const logger_1 = require("./logger");
const tsServerClient_1 = require("./tsServerClient");
const utils = require("./utils");
class Server {
    constructor(options) {
        this.options = options;
        this.logger = logger_1.createLogger(options.logFile);
        this.connection = vscode_languageserver_1.createConnection();
        this.registerLspEvents();
    }
    registerLspEvents() {
        this.connection.onInitialize(this._onInitialize.bind(this));
        this.connection.onDidOpenTextDocument(this.onDidOpenTextDocument.bind(this));
        this.connection.onDidSaveTextDocument(this.onDidSaveTextDocument.bind(this));
        this.connection.onDidCloseTextDocument(this.onDidCloseTextDocument.bind(this));
        this.connection.onDidChangeTextDocument(this.onDidChangeTextDocument.bind(this));
        this.connection.onDefinition(this.onDefinition.bind(this));
        this.connection.onDocumentSymbol(this.onDocumentSymbol.bind(this));
        this.connection.onCompletion(this.onCompletion.bind(this));
        this.connection.onHover(this.onHover.bind(this));
        this.connection.onReferences(this.onReferences.bind(this));
        this.connection.onWorkspaceSymbol(this.onWorkspaceSymbol.bind(this));
    }
    listen() {
        this.logger.info('Server.listen()');
        this.connection.listen();
    }
    _onInitialize(params) {
        this.logger.info('Server._onInitialize()', params);
        // TODO: validate rootPath and rootUri
        this.initializeParams = params;
        this.tsServerClient = new tsServerClient_1.TsServerClient({
            tsserverPath: this.options.tsserverPath,
            logFile: this.options.tsserverLogFile,
            logger: this.logger
        });
        this.tsServerClient.start();
        this.initializeResult = {
            capabilities: {
                textDocumentSync: vscode_languageserver_1.TextDocumentSyncKind.Full,
                completionProvider: {
                    triggerCharacters: ['.'],
                    resolveProvider: false
                },
                definitionProvider: true,
                documentSymbolProvider: true,
                hoverProvider: true,
                referencesProvider: true,
                workspaceSymbolProvider: true
            }
        };
        this.logger.info('Server._onInitialize() result', this.initializeResult);
        return this.initializeResult;
    }
    onDidOpenTextDocument(params) {
        const path = utils.uriToPath(params.textDocument.uri);
        this._lastFile = path;
        this.logger.info('Server.onDidOpenTextDocument()', params, path);
        this.tsServerClient.sendOpen(path);
    }
    onDidCloseTextDocument(params) {
        const path = utils.uriToPath(params.textDocument.uri);
        this.logger.info('Server.onDidCloseTextDocument()', params, path);
        this.tsServerClient.sendClose(path);
    }
    onDidChangeTextDocument(params) {
        const path = utils.uriToPath(params.textDocument.uri), tempPath = tempy.file({ extension: 'ts' });
        this._lastFile = path;
        this.logger.info('Server.onDidCloseTextDocument()', params, path, tempPath);
        fs.writeFileSync(tempPath, params.contentChanges[0].text, 'utf8');
        this.tsServerClient.sendReload(path, tempPath);
    }
    onDidSaveTextDocument(params) {
        const path = utils.uriToPath(params.textDocument.uri), tempPath = tempy.file({ extension: 'ts' });
        this._lastFile = path;
        this.logger.info('Server.onDidChangeTextDocument()', params, path, tempPath);
        fs.writeFileSync(tempPath, params.contentChanges[0].text, 'utf8');
        this.tsServerClient.sendSaveTo(path, tempPath);
    }
    onDefinition(params) {
        const path = utils.uriToPath(params.textDocument.uri);
        this.logger.info('Server.onDefinition()', params, path);
        return this.tsServerClient.sendDefinition(path, params.position.line + 1, params.position.character + 1)
            .then(result => {
            return result.body
                .map(fileSpan => utils.tsServerFileSpanToLspLocation(fileSpan));
        });
    }
    onDocumentSymbol(params) {
        const path = utils.uriToPath(params.textDocument.uri);
        this.logger.info('Server.onDocumentSymbol()', params, path);
        return this.tsServerClient.sendNavTree(path)
            .then(result => {
            // walk navtree in pre-order
            const symbols = [], stack = [];
            if (!!result.body) {
                stack.push(result.body);
            }
            while (stack.length > 0) {
                const item = stack.pop(), span = item.spans[0];
                if (span) {
                    symbols.push({
                        location: {
                            uri: params.textDocument.uri,
                            range: {
                                start: utils.tsServerLocationToLspPosition(span.start),
                                end: utils.tsServerLocationToLspPosition(span.end)
                            }
                        },
                        // TODO: for now make everything as variable for unknown
                        kind: utils.symbolKindsMapping[item.kind] || vscode_languageserver_1.SymbolKind.Variable,
                        name: item.text
                    });
                }
                if (item.childItems) {
                    item.childItems.forEach(childItem => stack.push(childItem));
                }
            }
            return symbols;
        });
    }
    onCompletion(params) {
        const path = utils.uriToPath(params.textDocument.uri);
        this.logger.info('Server.onCompletion()', params, path);
        return this.tsServerClient.sendCompletions(path, params.position.line + 1, params.position.character + 1, '')
            .then(result => {
            return {
                isIncomplete: false,
                items: result.body
                    .map(item => {
                    return {
                        label: item.name,
                        kind: utils.completionKindsMapping[item.kind]
                    };
                })
            };
        });
    }
    onHover(params) {
        const path = utils.uriToPath(params.textDocument.uri);
        this.logger.info('Server.onHover()', params, path);
        return this.tsServerClient.sendQuickInfo(path, params.position.line + 1, params.position.character + 1)
            .then(result => {
            return {
                contents: result.body.displayString,
                range: {
                    start: utils.tsServerLocationToLspPosition(result.body.start),
                    end: utils.tsServerLocationToLspPosition(result.body.end)
                }
            };
        });
    }
    onReferences(params) {
        const path = utils.uriToPath(params.textDocument.uri);
        this.logger.info('Server.onReferences()', params, path);
        return this.tsServerClient.sendReferences(path, params.position.line + 1, params.position.character + 1)
            .then(result => {
            return result.body.refs
                .map(fileSpan => utils.tsServerFileSpanToLspLocation(fileSpan));
        });
    }
    onWorkspaceSymbol(params) {
        return this.tsServerClient.sendNavTo(params.query, this._lastFile)
            .then((result) => {
            return result.body.map(item => {
                return {
                    location: {
                        uri: utils.pathToUri(item.file),
                        range: {
                            start: utils.tsServerLocationToLspPosition(item.start),
                            end: utils.tsServerLocationToLspPosition(item.end)
                        }
                    },
                    // TODO: for now make everything as variable for unknown
                    kind: utils.symbolKindsMapping[item.kind] || vscode_languageserver_1.SymbolKind.Variable,
                    name: item.name
                };
            });
        });
    }
}
exports.Server = Server;
