"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cp = require("child_process");
const utils = require("./utils");
class TsServerClient {
    constructor(options) {
        this.options = options;
        this.started = false;
        this.seq = 0;
        this.buffer = '';
        this.deferreds = {};
    }
    get logger() {
        return this.options.logger;
    }
    start() {
        if (this.started) {
            return;
        }
        this.logger.info('TsServerClient.start(): starting', { tsserverPath: this.options.tsserverPath, logFile: this.options.logFile });
        this.cp = cp.spawn(this.options.tsserverPath, this.options.logFile ? ['-logToFile', 'true', '-file', this.options.logFile] : []);
        this.cp.stdout.setEncoding('utf8');
        this.cp.stdout.addListener('data', this.onTsServerData.bind(this));
        this.logger.info('TsServerClient.start(): started');
    }
    onTsServerData(data) {
        this.buffer += data;
        while (true) {
            if (!this.header) {
                this.logger.info('we have not parsed the header yet');
                const headerEndIndex = this.buffer.indexOf('\r\n\r\n');
                if (headerEndIndex >= 0) {
                    this.logger.info('we have full header');
                    this.header = {};
                    this.buffer.substring(0, headerEndIndex)
                        .split('\r\n')
                        .forEach(header => {
                        const kvp = header.split(':').map(values => values.trim());
                        if (kvp[0].toUpperCase() === 'CONTENT-LENGTH') {
                            this.header['Content-Length'] = parseInt(kvp[1]);
                        }
                        else {
                            this.header[kvp[0]] = kvp[1];
                        }
                    });
                    this.buffer = this.buffer.substring(headerEndIndex + 4);
                }
                else {
                    this.logger.info('wait for next buffer to arrive');
                    break;
                }
            }
            else {
                const contentLength = this.header['Content-Length'] + 1;
                if (this.buffer.length >= contentLength) {
                    const messageString = this.buffer.substring(0, contentLength);
                    this.logger.info('we have the full message', this.header, messageString);
                    const message = JSON.parse(messageString);
                    this.buffer = this.buffer.substring(contentLength);
                    this.header = null;
                    this.processMessage(message);
                    if (this.buffer.length > 0) {
                        this.logger.info('we have more data in the buffer so try parsing the new headers from top', this.buffer);
                        continue;
                    }
                    else {
                        this.logger.info('we are done processing the message here so stop');
                        break;
                    }
                }
                else {
                    this.logger.info('we do not have the entire message body, so wait for the next bufer');
                    break;
                }
            }
        }
    }
    sendRequest(command, notification, args) {
        this.seq = this.seq + 1;
        let request = {
            command,
            seq: this.seq,
            type: 'request'
        };
        if (args) {
            request.arguments = args;
        }
        const serializedRequest = JSON.stringify(request) + '\n';
        this.logger.info('---->', serializedRequest);
        if (!this.cp.stdin.write(serializedRequest)) {
            this.logger.info('not flushed');
        }
        return notification ? undefined : (this.deferreds[this.seq] = new utils.Deferred()).promise;
    }
    processMessage(message) {
        this.logger.info(Object.keys(message));
        const deferred = this.deferreds[message.request_seq];
        this.logger.info('has deferred', !!deferred, message.type, message.command, message.request_seq, Object.keys(this.deferreds));
        if (deferred) {
            if (message.success) {
                this.deferreds[message.request_seq].resolve(message);
            }
            else {
                this.deferreds[message.request_seq].reject(message);
            }
            delete this.deferreds[message.request_seq];
        }
    }
    sendOpen(file) {
        const args = { file };
        this.logger.info('TsServerClient.sendOpen()', file);
        this.sendRequest('open', true, args);
    }
    sendClose(file) {
        const args = { file };
        this.logger.info('TsServerClient.sendClose()', file);
        this.sendRequest('close', true, args);
    }
    sendSaveTo(file, tmpfile) {
        const args = { file, tmpfile };
        this.logger.info('TsServerClient.sendSaveTo()', file, tmpfile);
        this.sendRequest('saveto', true, args);
    }
    sendReload(file, tmpfile) {
        const args = { file, tmpfile };
        this.logger.info('TsServerClient.sendReload()', file, tmpfile);
        this.sendRequest('reload', true, args);
    }
    sendDefinition(file, line, offset) {
        const args = { file, line, offset };
        this.logger.info('TsServerClient.sendDefinition()', args);
        return this.sendRequest('definition', false, args);
    }
    sendNavTo(searchValue, file) {
        let args = { searchValue, file };
        this.logger.info('TsServerClient.sendNavTo()', args);
        return this.sendRequest('navto', false, args);
    }
    sendNavTree(file) {
        const args = { file };
        this.logger.info('TsServerClient.sendNavTree()', args);
        return this.sendRequest('navtree', false, args);
    }
    sendCompletions(file, line, offset, prefix) {
        const args = { file, line, offset, prefix };
        this.logger.info('TsServerClient.sendCompletions()', args);
        return this.sendRequest('completions', false, args);
    }
    sendQuickInfo(file, line, offset) {
        const args = { file, line, offset };
        this.logger.info('TsServerClient.sendQuickInfo()', args);
        return this.sendRequest('quickinfo', false, args);
    }
    sendReferences(file, line, offset) {
        const args = { file, line, offset };
        this.logger.info('TsServerClient.sendReferences()', args);
        return this.sendRequest('references', false, args);
    }
}
exports.TsServerClient = TsServerClient;
